@@ -1,27 +1,14 @@
 #!/bin/sh
 #
-# Copyright (C) 2013 - Christian Babeux <christian.babeux@efficios.com>
+# Copyright (C) 2013 Christian Babeux <christian.babeux@efficios.com>
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; only version 2
-# of the License.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# SPDX-License-Identifier: GPL-2.0-only
 #
 
 # System binaries paths.
 CAT_BIN="cat"
 PGREP_BIN="pgrep"
 MKDIR_BIN="mkdir"
-LTTNG_BIN="lttng"
 
 # Session name
 SESSION_NAME="coredump-handler"
