@@ -670,6 +670,8 @@ struct lttng_consumer_stream *consumer_stream_create(
 		goto error;
 	}
 
+	stream->send_node = (typeof(stream->send_node))
+			CDS_LIST_HEAD_INIT(stream->send_node);
 	stream->chan = channel;
 	stream->key = stream_key;
 	stream->trace_chunk = trace_chunk;
@@ -1059,6 +1061,8 @@ void consumer_stream_destroy(struct lttng_consumer_stream *stream,
 {
 	assert(stream);
 
+	cds_list_del_init(&stream->send_node);
+
 	/* Stream is in monitor mode. */
 	if (stream->monitor) {
 		struct lttng_consumer_channel *free_chan = NULL;
@@ -1091,6 +1095,7 @@ void consumer_stream_destroy(struct lttng_consumer_stream *stream,
 			 * If the stream is not visible globally, this needs to be done
 			 * outside of the consumer data lock section.
 			 */
+			destroy_close_stream(stream);
 			free_chan = unref_channel(stream);
 		}
 
