@@ -48,7 +48,7 @@ int lttng_dynamic_buffer_append(struct lttng_dynamic_buffer *buffer,
  */
 LTTNG_HIDDEN
 int lttng_dynamic_buffer_append_buffer(struct lttng_dynamic_buffer *dst_buffer,
-		struct lttng_dynamic_buffer *src_buffer);
+		const struct lttng_dynamic_buffer *src_buffer);
 
 /*
  * Performs the same action as lttng_dynamic_buffer_append(), but using a
